"""
    This module contains gardeshpay.ir API client.
"""

import requests

from gp_pay.utils.reqres import get_json
from gp_pay.api.urls import API_URL_SEND, API_URL_VERIFY


class GPClient:
    def __init__(self, token):
        self.token = token

    def payment(self, amount, callback, invoice_number, invoice_date, description, mobile=None, email=None):
        if not isinstance(amount, int):
            raise TypeError('\'amount\' should be integer.')
        if amount < 1000:
            raise ValueError('\'amount\' value must be >= 1000.')

        if not isinstance(callback, str):
            raise TypeError('\'redirect_url\' should be string.')

        if invoice_number and not isinstance(invoice_number, str):
            raise TypeError('\'factor_number\' should be string.')
        if invoice_date and not isinstance(invoice_date, str):
            raise TypeError('\'factor_number\' should be string.')
        if mobile:
            if not isinstance(mobile, str):
                raise TypeError('\'mobile\' should be string.')
            if len(mobile) != 11:
                raise ValueError('\'mobile\' should be 11 characters.')
            if not mobile.startswith('09'):
                raise ValueError('\'mobile\' should start with \'09\'.')

        if description:
            if not isinstance(description, str):
                raise TypeError('\'description\' should be string.')

        init_data = {
            'token': self.token,
            'amount': amount,
            'callback': callback,
            'invoiceNumber': invoice_number if invoice_number is not None else 0,
            'invoiceDate': invoice_date,
            'description': description
        }
        if mobile:
            init_data['mobile'] = mobile
        if email:
            init_data['email'] = email

        response = get_json(requests.post(API_URL_SEND, data=init_data))
        if response['success']:
            return response['data']['url'] + '/' + response['data']['token']
        else:
            return response['errors'][0]['message']

    def verify_transaction(self, tracking_number):
        if not isinstance(tracking_number, str):
            raise TypeError('tracking number should be of string type.')

        verify_data = {
            'token': self.token,
            'trackingNumber': tracking_number
        }

        response = get_json(requests.post(API_URL_VERIFY, data=verify_data))
        if response['success']:
            return response['amount']
